#include "iki.h"
#include "private-iki.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_f_iki_content_is_
  f_status_t f_iki_content_is(const f_string_static_t content, const f_string_static_t quote) {
    #ifndef _di_level_0_parameter_checking_
      if (!quote.used) return F_status_set_error(F_parameter);
      if (quote.string[0] != f_iki_syntax_quote_single_s.string[0] && quote.string[0] != f_iki_syntax_quote_double_s.string[0] && quote.string[0] != f_iki_syntax_quote_grave_s.string[0]) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (!content.used) return F_data_not;

    return private_f_iki_content_partial_is(content.string, content.used, quote.string[0]);
  }
#endif // _di_f_iki_content_is_

#ifndef _di_f_iki_content_partial_is_
  f_status_t f_iki_content_partial_is(const f_string_static_t content, const f_range_t range, const f_string_static_t quote) {
    #ifndef _di_level_0_parameter_checking_
      if (!quote.used) return F_status_set_error(F_parameter);
      if (quote.string[0] != f_iki_syntax_quote_single_s.string[0] && quote.string[0] != f_iki_syntax_quote_double_s.string[0] && quote.string[0] != f_iki_syntax_quote_grave_s.string[0]) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (!content.used) return F_data_not;
    if (range.start > range.stop) return F_data_not_stop;
    if (range.start >= content.used) return F_data_not_eos;

    return private_f_iki_content_partial_is(content.string + range.start, (range.stop - range.start) + 1, quote.string[0]);
  }
#endif // _di_f_iki_content_partial_is_

#ifndef _di_f_iki_eki_read_
  void f_iki_eki_read(f_string_static_t * const buffer, f_range_t * const range, f_iki_eki_t * const data, f_state_t * const state) {
    #ifndef _di_level_0_parameter_checking_
      if (!state) return;

      if (!buffer || !range || !data) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_0_parameter_checking_

    if (!buffer->used) {
      state->status = F_data_not;

      return;
    }

    if (range->start > range->stop) {
      state->status = F_data_not_stop;

      return;
    }

    if (range->start >= buffer->used) {
      state->status = F_data_not_eos;

      return;
    }

    f_number_unsigned_t width_max = 0;

    if (width_max > buffer->used - range->start) {
      width_max = buffer->used - range->start;
    }

    f_range_t content_range = f_range_t_initialize;
    f_range_t found_vocabulary = f_range_t_initialize;
    f_number_unsigned_t i = 0;
    f_number_unsigned_t content_slash_delimits = 0;
    f_number_unsigned_t content_slash_first = 0;
    f_number_unsigned_t content_slash_total = 0;
    f_number_unsigned_t found_content = 0;
    f_number_unsigned_t vocabulary_count = 0;
    f_number_unsigned_t vocabulary_slash_first = 0;
    const f_number_unsigned_t delimits_used = data->delimits.used;

    uint8_t mode = 0x0; // 0x0 = nothing, 0x1 = separator found, 0x2 = possible multiple vocabulary names, 0x4 = completed multiple vocabulary names.
    uint8_t quote = 0;
    uint8_t wrapped = 0x0; // 0x0 = not wrapped, 0x1 = wrapped, 0x2 = valid wrapped.

    do {

      // Find the start of the vocabulary name.
      while (range->start <= range->stop && range->start < buffer->used) {

        if (state->interrupt) {
          state->interrupt((void * const) state, 0);
          if (F_status_set_fine(state->status) == F_interrupt) break;
        }

        width_max = buffer->used - range->start;

        state->status = f_utf_is_word_dash_plus(buffer->string + range->start, width_max, F_false);

        if (F_status_is_error(state->status)) {
          if (F_status_set_fine(state->status) == F_utf_fragment || F_status_set_fine(state->status) == F_complete_not_utf) {
            if (state->flag & f_iki_state_flag_utf_fail_on_valid_not_e) break;

            state->status = F_false;
          }
          else {
            break;
          }
        }

        if (state->status == F_true) {
          if (!wrapped) {
            found_vocabulary.start = range->start;
            found_vocabulary.stop = range->start;
          }

          state->status = f_utf_buffer_increment(*buffer, range, 1);
          if (F_status_is_error(state->status)) break;

          state->status = F_true;

          break;
        }

        // Wrapped must be followed by a valid vocabulary name.
        if (buffer->string[range->start] == f_iki_syntax_wrap_open_s.string[0]) {
          found_vocabulary.start = range->start;
          found_vocabulary.stop = range->start;

          wrapped = 0x1;
        }
        else if (wrapped) {
          wrapped = 0x0;
        }

        state->status = f_utf_buffer_increment(*buffer, range, 1);
        if (F_status_is_error(state->status)) break;
      } // while

      // Find the end of the vocabulary name.
      while (F_status_is_error_not(state->status) && range->start <= range->stop && range->start < buffer->used) {

        if (state->interrupt) {
          state->interrupt((void * const) state, 0);
          if (F_status_set_fine(state->status) == F_interrupt) break;
        }

        if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
          ++range->start;

          continue;
        }

        if (buffer->string[range->start] == f_iki_syntax_separator_s.string[0]) {
          do {
            state->status = f_utf_buffer_increment(*buffer, range, 1);
          } while (F_status_is_error_not(state->status) && range->start <= range->stop && range->start < buffer->used && buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]);

          if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used) break;

          // Found a valid vocabulary name.
          if (buffer->string[range->start] == f_iki_syntax_quote_single_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_double_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_grave_s.string[0]) {
            state->status = F_true;
            quote = buffer->string[range->start];
            vocabulary_count = 1;
          }
          else {
            mode &= ~0x6;
            vocabulary_count = 0;

            do {
              if (state->interrupt) {
                state->interrupt((void * const) state, 0);
                if (F_status_set_fine(state->status) == F_interrupt) break;
              }

              if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
                ++range->start;

                continue;
              }

              if (buffer->string[range->start] == f_iki_syntax_quote_single_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_double_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_grave_s.string[0]) {
                if ((mode & 0x6) && (!wrapped || wrapped == 0x2)) {
                  if (mode & 0x4) {
                    mode &= ~0x2;
                  }
                  else {
                    mode |= 0x4;
                  }

                  state->status = F_true;
                  quote = buffer->string[range->start];
                  ++vocabulary_count;
                }
                else {
                  state->status = F_next;
                  mode &= ~0x6;
                }

                break;
              }

              if (buffer->string[range->start] == f_iki_syntax_slash_s.string[0]) {
                mode &= ~0x1;
                vocabulary_slash_first = range->start;

                // The slash only needs to be delimited if it were to otherwise be a valid vocabulary name.
                while (range->start <= range->stop && range->start < buffer->used) {

                  if (state->interrupt) {
                    state->interrupt((void * const) state, 0);
                    if (F_status_set_fine(state->status) == F_interrupt) break;
                  }

                  if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
                    ++range->start;

                    continue;
                  }

                  if (mode & 0x1) {

                    // Save delimit for a would-be valid IKI that is now delimited.
                    if (buffer->string[range->start] == f_iki_syntax_quote_single_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_double_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_grave_s.string[0]) {
                      state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &data->delimits.array, &data->delimits.used, &data->delimits.size);
                      if (F_status_is_error(state->status)) break;

                      data->delimits.array[data->delimits.used++] = vocabulary_slash_first;

                      ++range->start;
                    }

                    state->status = F_next;

                    break;
                  }

                  if (buffer->string[range->start] == f_iki_syntax_separator_s.string[0]) {
                    mode |= 0x1;
                  }
                  else if (buffer->string[range->start] != f_iki_syntax_slash_s.string[0]) {
                    state->status = F_next;

                    break;
                  }

                  state->status = f_utf_buffer_increment(*buffer, range, 1);
                  if (F_status_is_error(state->status)) break;
                } // while

                if (state->status == F_true) break;
                if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used || state->status == F_next) break;
              }
              else if (buffer->string[range->start] == f_iki_syntax_separator_s.string[0]) {
                if (!(mode & 0x2)) {
                  mode &= ~0x6;

                  break;
                }

                ++range->start;
                ++vocabulary_count;
                mode &= ~0x2;
                mode |= 0x4;

                continue;
              }
              else if (buffer->string[range->start] == f_iki_syntax_wrap_close_s.string[0]) {
                if (wrapped == 0x1) {
                  wrapped = 0x2;
                  found_vocabulary.stop = range->start;
                }
                else {
                  state->status = F_next;
                  mode &= ~0x6;

                  break;
                }
              }
              else {
                width_max = buffer->used - range->start;

                state->status = f_utf_is_word_dash_plus(buffer->string + range->start, width_max, F_false);

                if (F_status_is_error_not(state->status)) {
                  if (state->status != F_true) {
                    state->status = F_next;

                    break;
                  }
                }

                if (state->status == F_true) {
                  mode |= 0x2;
                  found_vocabulary.stop = range->start;
                }
              }

              state->status = f_utf_buffer_increment(*buffer, range, 1);
              if (F_status_is_error(state->status)) break;

            } while (F_status_is_error_not(state->status) && range->start <= range->stop && range->start < buffer->used);
          }

          break;
        }

        if (buffer->string[range->start] == f_iki_syntax_slash_s.string[0]) {
          mode &= ~0x1;
          vocabulary_slash_first = range->start;

          // The slash only needs to be delimited if it were to otherwise be a valid vocabulary name.
          while (range->start <= range->stop && range->start < buffer->used) {

            if (state->interrupt) {
              state->interrupt((void * const) state, 0);
              if (F_status_set_fine(state->status) == F_interrupt) break;
            }

            if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
              ++range->start;

              continue;
            }

            if (mode & 0x1) {

              // Save delimit for a would-be valid IKI that is now delimited.
              if (buffer->string[range->start] == f_iki_syntax_quote_single_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_double_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_grave_s.string[0]) {
                state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &data->delimits.array, &data->delimits.used, &data->delimits.size);
                if (F_status_is_error(state->status)) break;

                data->delimits.array[data->delimits.used++] = vocabulary_slash_first;

                ++range->start;
              }

              state->status = F_next;

              break;
            }

            if (buffer->string[range->start] == f_iki_syntax_separator_s.string[0]) {
              mode |= 0x1;
            }
            else if (buffer->string[range->start] != f_iki_syntax_slash_s.string[0]) {
              state->status = F_next;

              break;
            }

            state->status = f_utf_buffer_increment(*buffer, range, 1);
            if (F_status_is_error(state->status)) break;
          } // while

          if (state->status == F_true) break;
          if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used || state->status == F_next) break;
        }
        else if (buffer->string[range->start] == f_iki_syntax_wrap_open_s.string[0]) {
          state->status = F_next;

          break;
        }
        else if (wrapped == 0x1 && buffer->string[range->start] == f_iki_syntax_wrap_close_s.string[0]) {
          wrapped = 0x2;
          found_vocabulary.stop = range->start;
        }
        else if (wrapped == 0x2) {

          // Wrapped close must be immediately before a separator (ignoring any placeholders in between).
          state->status = F_next;

          break;
        }
        else {
          width_max = buffer->used - range->start;

          state->status = f_utf_is_word_dash_plus(buffer->string + range->start, width_max, F_false);

          if (F_status_is_error(state->status)) {
            if (F_status_set_fine(state->status) == F_utf_fragment || F_status_set_fine(state->status) == F_complete_not_utf) {
              if (state->flag & f_iki_state_flag_utf_fail_on_valid_not_e) break;

              state->status = F_false;
            }
            else {
              break;
            }
          }

          if (state->status == F_true) {
            found_vocabulary.stop = range->start;
          }
          else {

            // Not a valid IKI vocabulary name.
            state->status = f_utf_buffer_increment(*buffer, range, 1);
            if (F_status_is_error(state->status)) break;

            state->status = F_next;

            break;
          }
        }

        state->status = f_utf_buffer_increment(*buffer, range, 1);
        if (F_status_is_error(state->status)) break;
      } // while

      if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used) break;

      if (state->status == F_next) {
        quote = 0;
        wrapped = 0x0;

        continue;
      }

      // Process potentially valid content.
      if (quote) {
        found_content = ++range->start;

        while (range->start <= range->stop && range->start < buffer->used) {

          if (state->interrupt) {
            state->interrupt((void * const) state, 0);
            if (F_status_set_fine(state->status) == F_interrupt) break;
          }

          if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
            ++range->start;

            continue;
          }

          if (buffer->string[range->start] == quote) {
            state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->variable.array, &data->variable.used, &data->variable.size);
            if (F_status_is_error(state->status)) break;

            state->status = f_memory_array_increase(state->step_small, sizeof(f_ranges_t), (void **) &data->vocabularys.array, &data->vocabularys.used, &data->vocabularys.size);
            if (F_status_is_error(state->status)) break;

            state->status = f_memory_array_increase_by(vocabulary_count, sizeof(f_range_t), (void **) &data->vocabularys.array[data->vocabularys.used].array, &data->vocabularys.array[data->vocabularys.used].used, &data->vocabularys.array[data->vocabularys.used].size);
            if (F_status_is_error(state->status)) break;

            state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->content.array, &data->content.used, &data->content.size);
            if (F_status_is_error(state->status)) break;

            data->variable.array[data->variable.used].start = found_vocabulary.start;
            data->variable.array[data->variable.used++].stop = range->start;

            i = wrapped ? found_vocabulary.start + f_iki_syntax_wrap_open_s.used : found_vocabulary.start;

            for (data->vocabularys.array[data->vocabularys.used].used = 0; data->vocabularys.array[data->vocabularys.used].used < vocabulary_count; ) {

              if (state->interrupt) {
                state->interrupt((void * const) state, 0);
                if (F_status_set_fine(state->status) == F_interrupt) break;
              }

              data->vocabularys.array[data->vocabularys.used].array[data->vocabularys.array[data->vocabularys.used].used].start = i;

              while (i <= found_vocabulary.stop && buffer->string[i] != f_iki_syntax_separator_s.string[0] && buffer->string[i] != f_iki_syntax_wrap_close_s.string[0]) ++i;

              data->vocabularys.array[data->vocabularys.used].array[data->vocabularys.array[data->vocabularys.used].used++].stop = (buffer->string[i] == f_iki_syntax_separator_s.string[0] || buffer->string[i] == f_iki_syntax_wrap_close_s.string[0]) ? i - 1 : i;

              if (i <= found_vocabulary.stop && buffer->string[i] == f_iki_syntax_separator_s.string[0]) {
                if (i == found_vocabulary.stop) break;

                ++i;
              }
            } // for

            ++data->vocabularys.used;

            data->content.array[data->content.used].start = found_content;
            data->content.array[data->content.used++].stop = range->start - 1;

            if (++range->start > range->stop) {
              state->status = F_okay_stop;
            }
            else if (range->start >= buffer->used) {
              state->status = F_okay_eos;
            }
            else {
              state->status = F_okay;
            }

            return;
          }

          if (buffer->string[range->start] == f_iki_syntax_slash_s.string[0]) {
            content_slash_first = range->start;
            content_slash_total = 0;

            while (range->start <= range->stop && range->start < buffer->used) {

              if (state->interrupt) {
                state->interrupt((void * const) state, 0);
                if (F_status_set_fine(state->status) == F_interrupt) break;
              }

              if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
                ++range->start;

                continue;
              }

              if (buffer->string[range->start] == quote) {
                content_slash_delimits = content_slash_total / 2;
                content_range.start = 1;
                content_range.stop = 0;

                if (content_slash_total % 2) {
                  ++content_slash_delimits;
                }

                state->status = f_memory_array_increase_by(content_slash_delimits, sizeof(f_number_unsigned_t), (void **) &data->delimits.array, &data->delimits.used, &data->delimits.size);
                if (F_status_is_error(state->status)) break;

                content_range.start = content_slash_first;
                content_range.stop = range->stop;

                for (i = 0; i < content_slash_delimits; ) {

                  if (buffer->string[content_range.start] == f_iki_syntax_slash_s.string[0]) {
                    data->delimits.array[data->delimits.used++] = content_range.start;
                    ++i;
                  }

                  state->status = f_utf_buffer_increment(*buffer, (&content_range), 1);
                  if (F_status_is_error(state->status)) break;
                } // for

                if (F_status_is_error(state->status)) break;

                // Valid content's ending quote is not delimited, save and return.
                if (content_slash_total % 2 == 0) {
                  state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->variable.array, &data->variable.used, &data->variable.size);
                  if (F_status_is_error(state->status)) break;

                  state->status = f_memory_array_increase(state->step_small, sizeof(f_ranges_t), (void **) &data->vocabularys.array, &data->vocabularys.used, &data->vocabularys.size);
                  if (F_status_is_error(state->status)) break;

                  state->status = f_memory_array_increase(vocabulary_count, sizeof(f_range_t), (void **) &data->vocabularys.array[data->vocabularys.used].array, &data->vocabularys.array[data->vocabularys.used].used, &data->vocabularys.array[data->vocabularys.used].size);
                  if (F_status_is_error(state->status)) break;

                  state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->content.array, &data->content.used, &data->content.size);
                  if (F_status_is_error(state->status)) break;

                  data->variable.array[data->variable.used].start = found_vocabulary.start;
                  data->variable.array[data->variable.used++].stop = range->start;

                  i = wrapped ? found_vocabulary.start + f_iki_syntax_wrap_open_s.used : found_vocabulary.start;

                  for (data->vocabularys.array[data->vocabularys.used].used = 0; data->vocabularys.array[data->vocabularys.used].used < vocabulary_count; ) {

                    if (state->interrupt) {
                      state->interrupt((void * const) state, 0);
                      if (F_status_set_fine(state->status) == F_interrupt) break;
                    }

                    data->vocabularys.array[data->vocabularys.used].array[data->vocabularys.array[data->vocabularys.used].used].start = i;

                    while (i <= found_vocabulary.stop && buffer->string[i] != f_iki_syntax_separator_s.string[0] && buffer->string[i] != f_iki_syntax_wrap_close_s.string[0]) ++i;

                    data->vocabularys.array[data->vocabularys.used].array[data->vocabularys.array[data->vocabularys.used].used++].stop = (buffer->string[i] == f_iki_syntax_separator_s.string[0] || buffer->string[i] == f_iki_syntax_wrap_close_s.string[0]) ? i - 1 : i;

                    if (i <= found_vocabulary.stop && buffer->string[i] == f_iki_syntax_separator_s.string[0]) {
                      if (i == found_vocabulary.stop) break;

                      ++i;
                    }
                  } // for

                  ++data->vocabularys.used;

                  data->content.array[data->content.used].start = found_content;
                  data->content.array[data->content.used++].stop = range->start - 1;

                  if (++range->start > range->stop) {
                    state->status = F_okay_stop;
                  }
                  else if (range->start >= buffer->used) {
                    state->status = F_okay_eos;
                  }
                  else {
                    state->status = F_okay;
                  }

                  return;
                }

                break;
              }

              if (buffer->string[range->start] != f_iki_syntax_slash_s.string[0]) break;

              ++content_slash_total;

              state->status = f_utf_buffer_increment(*buffer, range, 1);
              if (F_status_is_error(state->status)) break;
            } // while

            if (F_status_is_error(state->status)) break;
          }

          state->status = f_utf_buffer_increment(*buffer, range, 1);
          if (F_status_is_error(state->status)) break;
        } // while

        quote = 0;
        wrapped = 0x0;
      }

      if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used) break;

      state->status = f_utf_buffer_increment(*buffer, range, 1);
      if (F_status_is_error(state->status)) break;

    } while (range->start <= range->stop && range->start < buffer->used);

    if (F_status_set_fine(state->status) == F_complete_not_utf_eos || F_status_set_fine(state->status) == F_complete_not_utf_stop) {
      if (!(state->flag & f_iki_state_flag_utf_fail_on_valid_not_e)) {
        state->status = F_status_set_fine(state->status);
      }
    }

    if (F_status_is_error(state->status)) {
      data->delimits.used = delimits_used;

      return;
    }

    state->status = (range->start > range->stop) ? F_data_not_stop : F_data_not_eos;
  }
#endif // _di_f_iki_eki_read_

#ifndef _di_f_iki_eki_write_
  void f_iki_eki_write(const f_string_statics_t vocabularys, const f_string_static_t content, const f_string_static_t quote, f_string_dynamic_t * const destination, f_state_t * const state) {
    #ifndef _di_level_1_parameter_checking_
      if (!state) return;

      if (!destination || !quote.used || (quote.string[0] != f_iki_syntax_quote_double_s.string[0] && quote.string[0] != f_iki_syntax_quote_grave_s.string[0] && quote.string[0] != f_iki_syntax_quote_single_s.string[0])) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_1_parameter_checking_

    if (!vocabularys.used) {
      state->status = F_status_set_error(F_data_not);

      return;
    }

    f_range_t range = f_range_t_initialize;
    f_number_unsigned_t i = 0;
    f_number_unsigned_t j = 0;
    f_number_unsigned_t total = 0;
    f_number_unsigned_t count = 0;

    for (; i < vocabularys.used; ++i) {

      if (!vocabularys.array[i].used) continue;

      range.start = 0;
      range.stop = vocabularys.array[i].used - 1;
      ++count;

      do {
        if (state->interrupt) {
          state->interrupt((void * const) state, 0);
          if (F_status_set_fine(state->status) == F_interrupt) break;
        }

        j = vocabularys.array[i].used - range.start;

        state->status = f_utf_is_word_dash_plus(vocabularys.array[i].string + range.start, j, F_false);
        if (F_status_is_error(state->status)) break;

        if (state->status == F_false) {
          state->status = F_status_set_error(F_syntax);

          break;
        }

        total += vocabularys.array[i].used;

        state->status = f_utf_buffer_increment(vocabularys.array[i], &range, 1);

      } while (F_status_is_error_not(state->status) && range.start < range.stop);
    } // for

    if (F_status_is_error(state->status)) return;

    if (!total) {
      state->status = F_status_set_error(F_data_not);

      return;
    }

    // Pre-allocate the expected size along with two extra slots for slashes to help reduce potential allocations.
    if (F_status_is_error_not(state->status)) {
      state->status = f_memory_array_increase_by(total + ((count - 1) * f_iki_syntax_separator_s.used) + content.used + f_iki_syntax_wrap_close_s.used + f_iki_syntax_wrap_open_s.used + f_iki_syntax_separator_s.used + (f_iki_syntax_slash_s.used * 2) + (quote.used * 2), sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
    }

    if (F_status_is_error_not(state->status) && (state->flag & f_iki_state_flag_force_wrap_e)) {
      state->status = f_string_dynamic_append(f_iki_syntax_wrap_open_s, destination);
    }

    total = count;
    count = 0;

    for (i = 0; F_status_is_error_not(state->status) && i < vocabularys.used; ++i) {

      if (!vocabularys.array[i].used) continue;

      state->status = f_string_dynamic_append(vocabularys.array[i], destination);

      if (F_status_is_error_not(state->status) && ++count < total) {
        state->status = f_string_dynamic_append(f_iki_syntax_separator_s, destination);
      }
    } // for

    if (F_status_is_error_not(state->status) && (state->flag & f_iki_state_flag_force_wrap_e)) {
      state->status = f_string_dynamic_append(f_iki_syntax_wrap_close_s, destination);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_string_dynamic_append(f_iki_syntax_separator_s, destination);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_string_dynamic_append(quote, destination);
    }

    if (F_status_is_error_not(state->status)) {
      total = 0;
      j = 0;
      range.start = 0;
      range.stop = content.used - 1;

      for (i = 0; i < content.used; ++i) {

        if (!content.string[i]) continue;

        if (content.string[i] == quote.string[0]) {
          state->status = f_memory_array_increase_by((i - range.start) + 1 + (total * f_iki_syntax_slash_s.used) + f_iki_syntax_slash_s.used + quote.used, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

          if (F_status_is_error_not(state->status) && i) {
            range.stop = i - 1;

            state->status = f_string_dynamic_partial_append(content, range, destination);
          }

          if (total) {
            for (j = 0; F_status_is_error_not(state->status) && j < total; ++j) {
              state->status = f_string_dynamic_append(f_iki_syntax_slash_s, destination);
            } // for
          }

          if (F_status_is_error_not(state->status)) {
            state->status = f_string_dynamic_append(f_iki_syntax_slash_s, destination);
          }

          if (F_status_is_error_not(state->status)) {
            state->status = f_string_dynamic_append(quote, destination);
          }

          if (F_status_is_error(state->status)) break;

          range.start = i + 1;
          range.stop = content.used - 1;
          total = 0;
        }
        else if (content.string[i] == f_iki_syntax_slash_s.string[0]) {
          ++total;
        }
        else if (total) {
          total = 0;
        }
      } // for

      if (F_status_is_error_not(state->status) && range.start <= range.stop) {
        state->status = f_string_dynamic_partial_append(content, range, destination);
      }

      // Slashes at the end must be escaped.
      if (F_status_is_error_not(state->status) && total) {
        state->status = f_memory_array_increase_by(total, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

        for (j = 0; F_status_is_error_not(state->status) && j < total; ++j) {
          state->status = f_string_dynamic_append(f_iki_syntax_slash_s, destination);
        } // for
      }
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_string_dynamic_append(quote, destination);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = F_okay;
    }
  }
#endif // _di_f_iki_eki_write_

#ifndef _di_f_iki_object_is_
  f_status_t f_iki_object_is(const f_string_static_t object) {

    if (!object.used) return F_data_not;

    return private_f_iki_object_partial_is(object.string, object.used);
  }
#endif // _di_f_iki_object_is_

#ifndef _di_f_iki_object_partial_is_
  f_status_t f_iki_object_partial_is(const f_string_static_t object, const f_range_t range) {

    if (!object.used) return F_data_not;
    if (range.start > range.stop) return F_data_not_stop;
    if (range.start >= object.used) return F_data_not_eos;

    return private_f_iki_object_partial_is(object.string + range.start, (range.stop - range.start) + 1);
  }
#endif // _di_f_iki_object_partial_is_

#ifndef _di_f_iki_read_
  void f_iki_read(f_string_static_t * const buffer, f_range_t * const range, f_iki_data_t * const data, f_state_t * const state) {
    #ifndef _di_level_0_parameter_checking_
      if (!state) return;

      if (!buffer || !range || !data) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_0_parameter_checking_

    if (!buffer->used) {
      state->status = F_data_not;

      return;
    }

    if (range->start > range->stop) {
      state->status = F_data_not_stop;

      return;
    }

    if (range->start >= buffer->used) {
      state->status = F_data_not_eos;

      return;
    }

    f_number_unsigned_t width_max = 0;

    if (width_max > buffer->used - range->start) {
      width_max = buffer->used - range->start;
    }

    f_range_t content_range = f_range_t_initialize;
    f_range_t found_vocabulary = f_range_t_initialize;
    f_number_unsigned_t i = 0;
    f_number_unsigned_t content_slash_delimits = 0;
    f_number_unsigned_t content_slash_first = 0;
    f_number_unsigned_t content_slash_total = 0;
    f_number_unsigned_t found_content = 0;
    f_number_unsigned_t vocabulary_slash_first = 0;
    const f_number_unsigned_t delimits_used = data->delimits.used;


    uint8_t quote = 0;
    uint8_t separator_found = F_false;
    uint8_t wrapped = 0x0; // 0x0 (false) = not wrapped, 0x1 (true) = wrapped, 0x2 = valid wrapped.

    do {

      // Find the start of the vocabulary name.
      while (range->start <= range->stop && range->start < buffer->used) {

        if (state->interrupt) {
          state->interrupt((void * const) state, 0);
          if (F_status_set_fine(state->status) == F_interrupt) break;
        }

        width_max = buffer->used - range->start;

        state->status = f_utf_is_word_dash_plus(buffer->string + range->start, width_max, F_false);

        if (F_status_is_error(state->status)) {
          if (F_status_set_fine(state->status) == F_utf_fragment || F_status_set_fine(state->status) == F_complete_not_utf) {
            if (state->flag & f_iki_state_flag_utf_fail_on_valid_not_e) break;

            state->status = F_false;
          }
          else {
            break;
          }
        }

        if (state->status == F_true) {
          if (!wrapped) {
            found_vocabulary.start = range->start;
            found_vocabulary.stop = range->start;
          }

          state->status = f_utf_buffer_increment(*buffer, range, 1);
          if (F_status_is_error(state->status)) break;

          state->status = F_true;

          break;
        }

        // Wrapped must be followed by a valid vocabulary name.
        if (buffer->string[range->start] == f_iki_syntax_wrap_open_s.string[0]) {
          found_vocabulary.start = range->start;
          found_vocabulary.stop = range->start;

          wrapped = 0x1;
        }
        else if (wrapped) {
          wrapped = 0x0;
        }

        state->status = f_utf_buffer_increment(*buffer, range, 1);
        if (F_status_is_error(state->status)) break;
      } // while

      // Find the end of the vocabulary name.
      while (F_status_is_error_not(state->status) && range->start <= range->stop && range->start < buffer->used) {

        if (state->interrupt) {
          state->interrupt((void * const) state, 0);
          if (F_status_set_fine(state->status) == F_interrupt) break;
        }

        if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
          ++range->start;

          continue;
        }

        if (buffer->string[range->start] == f_iki_syntax_separator_s.string[0]) {

          // Wrapped must close in a "wrap close" before the separator is matched.
          if (wrapped == 0x1) {
            state->status = F_next;

            break;
          }

          do {
            state->status = f_utf_buffer_increment(*buffer, range, 1);
          } while (F_status_is_error_not(state->status) && range->start <= range->stop && range->start < buffer->used && buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]);

          if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used) break;

          // Found a valid vocabulary name.
          if (buffer->string[range->start] == f_iki_syntax_quote_single_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_double_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_grave_s.string[0]) {
            state->status = F_true;
            quote = buffer->string[range->start];
          }
          else {
            state->status = F_next;
          }

          break;
        }

        if (buffer->string[range->start] == f_iki_syntax_slash_s.string[0]) {
          separator_found = F_false;
          vocabulary_slash_first = range->start;

          // The slash only needs to be delimited if it were to otherwise be a valid vocabulary name.
          while (range->start <= range->stop && range->start < buffer->used) {

            if (state->interrupt) {
              state->interrupt((void * const) state, 0);
              if (F_status_set_fine(state->status) == F_interrupt) break;
            }

            if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
              ++range->start;

              continue;
            }

            if (separator_found) {

              // Save delimit for a would-be valid IKI that is now delimited.
              if (buffer->string[range->start] == f_iki_syntax_quote_single_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_double_s.string[0] || buffer->string[range->start] == f_iki_syntax_quote_grave_s.string[0]) {
                state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &data->delimits.array, &data->delimits.used, &data->delimits.size);
                if (F_status_is_error(state->status)) break;

                data->delimits.array[data->delimits.used++] = vocabulary_slash_first;

                ++range->start;
              }

              state->status = F_next;

              break;
            }

            if (buffer->string[range->start] == f_iki_syntax_separator_s.string[0]) {
              separator_found = F_true;
            }
            else if (buffer->string[range->start] != f_iki_syntax_slash_s.string[0]) {
              state->status = F_next;

              break;
            }

            state->status = f_utf_buffer_increment(*buffer, range, 1);
            if (F_status_is_error(state->status)) break;
          } // while

          if (state->status == F_true) break;
          if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used || state->status == F_next) break;
        }
        else if (buffer->string[range->start] == f_iki_syntax_wrap_open_s.string[0]) {
          state->status = F_next;

          break;
        }
        else if (wrapped == 0x1 && buffer->string[range->start] == f_iki_syntax_wrap_close_s.string[0]) {
          wrapped = 0x2;
          found_vocabulary.stop = range->start;
        }
        else if (wrapped == 0x2) {

          // Wrapped close must be immediately before a separator (ignoring any placeholders in between).
          state->status = F_next;

          break;
        }
        else {
          width_max = buffer->used - range->start;

          state->status = f_utf_is_word_dash_plus(buffer->string + range->start, width_max, F_false);

          if (F_status_is_error(state->status)) {
            if (F_status_set_fine(state->status) == F_utf_fragment || F_status_set_fine(state->status) == F_complete_not_utf) {
              if (state->flag & f_iki_state_flag_utf_fail_on_valid_not_e) break;

              state->status = F_false;
            }
            else {
              break;
            }
          }

          if (state->status == F_true) {
            found_vocabulary.stop = range->start;
          }
          else {

            // Not a valid IKI vocabulary name.
            state->status = f_utf_buffer_increment(*buffer, range, 1);
            if (F_status_is_error(state->status)) break;

            state->status = F_next;

            break;
          }
        }

        state->status = f_utf_buffer_increment(*buffer, range, 1);
        if (F_status_is_error(state->status)) break;
      } // while

      if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used) break;

      if (state->status == F_next) {
        quote = 0;
        wrapped = 0x0;

        continue;
      }

      // Process potentially valid content.
      if (quote) {
        found_content = ++range->start;

        while (range->start <= range->stop && range->start < buffer->used) {

          if (state->interrupt) {
            state->interrupt((void * const) state, 0);
            if (F_status_set_fine(state->status) == F_interrupt) break;
          }

          if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
            ++range->start;

            continue;
          }

          if (buffer->string[range->start] == quote) {
            state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->variable.array, &data->variable.used, &data->variable.size);
            if (F_status_is_error(state->status)) break;

            state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->vocabulary.array, &data->vocabulary.used, &data->vocabulary.size);
            if (F_status_is_error(state->status)) break;

            state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->content.array, &data->content.used, &data->content.size);
            if (F_status_is_error(state->status)) break;

            data->variable.array[data->variable.used].start = found_vocabulary.start;
            data->variable.array[data->variable.used++].stop = range->start;

            data->vocabulary.array[data->vocabulary.used].start = wrapped ? found_vocabulary.start + f_iki_syntax_wrap_open_s.used : found_vocabulary.start;
            data->vocabulary.array[data->vocabulary.used++].stop = wrapped ? found_vocabulary.stop - f_iki_syntax_wrap_close_s.used : found_vocabulary.stop;

            data->content.array[data->content.used].start = found_content;
            data->content.array[data->content.used++].stop = range->start - 1;

            if (++range->start > range->stop) {
              state->status = F_okay_stop;
            }
            else if (range->start >= buffer->used) {
              state->status = F_okay_eos;
            }
            else {
              state->status = F_okay;
            }

            return;
          }

          if (buffer->string[range->start] == f_iki_syntax_slash_s.string[0]) {
            content_slash_first = range->start;
            content_slash_total = 0;

            while (range->start <= range->stop && range->start < buffer->used) {

              if (state->interrupt) {
                state->interrupt((void * const) state, 0);
                if (F_status_set_fine(state->status) == F_interrupt) break;
              }

              if (buffer->string[range->start] == f_iki_syntax_placeholder_s.string[0]) {
                ++range->start;

                continue;
              }

              if (buffer->string[range->start] == quote) {
                content_slash_delimits = content_slash_total / 2;
                content_range.start = 1;
                content_range.stop = 0;

                if (content_slash_total % 2) {
                  ++content_slash_delimits;
                }

                state->status = f_memory_array_increase_by(content_slash_delimits, sizeof(f_number_unsigned_t), (void **) &data->delimits.array, &data->delimits.used, &data->delimits.size);
                if (F_status_is_error(state->status)) break;

                content_range.start = content_slash_first;
                content_range.stop = range->stop;

                for (i = 0; i < content_slash_delimits; ) {

                  if (buffer->string[content_range.start] == f_iki_syntax_slash_s.string[0]) {
                    data->delimits.array[data->delimits.used++] = content_range.start;
                    ++i;
                  }

                  state->status = f_utf_buffer_increment(*buffer, (&content_range), 1);
                  if (F_status_is_error(state->status)) break;
                } // for

                if (F_status_is_error(state->status)) break;

                // Valid content's ending quote is not delimited, save and return.
                if (content_slash_total % 2 == 0) {
                  state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->variable.array, &data->variable.used, &data->variable.size);
                  if (F_status_is_error(state->status)) break;

                  state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->vocabulary.array, &data->vocabulary.used, &data->vocabulary.size);
                  if (F_status_is_error(state->status)) break;

                  state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &data->content.array, &data->content.used, &data->content.size);
                  if (F_status_is_error(state->status)) break;

                  data->variable.array[data->variable.used].start = found_vocabulary.start;
                  data->variable.array[data->variable.used++].stop = range->start;

                  data->vocabulary.array[data->vocabulary.used].start = wrapped ? found_vocabulary.start + f_iki_syntax_wrap_open_s.used : found_vocabulary.start;
                  data->vocabulary.array[data->vocabulary.used++].stop = wrapped ? found_vocabulary.stop - f_iki_syntax_wrap_close_s.used : found_vocabulary.stop;

                  data->content.array[data->content.used].start = found_content;
                  data->content.array[data->content.used++].stop = range->start - 1;

                  if (++range->start > range->stop) {
                    state->status = F_okay_stop;
                  }
                  else if (range->start >= buffer->used) {
                    state->status = F_okay_eos;
                  }
                  else {
                    state->status = F_okay;
                  }

                  return;
                }

                break;
              }

              if (buffer->string[range->start] != f_iki_syntax_slash_s.string[0]) break;

              ++content_slash_total;

              state->status = f_utf_buffer_increment(*buffer, range, 1);
              if (F_status_is_error(state->status)) break;
            } // while

            if (F_status_is_error(state->status)) break;
          }

          state->status = f_utf_buffer_increment(*buffer, range, 1);
          if (F_status_is_error(state->status)) break;
        } // while

        quote = 0;
        wrapped = 0x0;
      }

      if (F_status_is_error(state->status) || range->start > range->stop || range->start >= buffer->used) break;

      state->status = f_utf_buffer_increment(*buffer, range, 1);
      if (F_status_is_error(state->status)) break;

    } while (range->start <= range->stop && range->start < buffer->used);

    if (F_status_set_fine(state->status) == F_complete_not_utf_eos || F_status_set_fine(state->status) == F_complete_not_utf_stop) {
      if (!(state->flag & f_iki_state_flag_utf_fail_on_valid_not_e)) {
        state->status = F_status_set_fine(state->status);
      }
    }

    if (F_status_is_error(state->status)) {
      data->delimits.used = delimits_used;

      return;
    }

    state->status = (range->start > range->stop) ? F_data_not_stop : F_data_not_eos;
  }
#endif // _di_f_iki_read_

#ifndef _di_f_iki_write_
  void f_iki_write(const f_string_static_t vocabulary, const f_string_static_t content, const f_string_static_t quote, f_string_dynamic_t * const destination, f_state_t * const state) {
    #ifndef _di_level_1_parameter_checking_
      if (!state) return;

      if (!destination || !quote.used || (quote.string[0] != f_iki_syntax_quote_double_s.string[0] && quote.string[0] != f_iki_syntax_quote_grave_s.string[0] && quote.string[0] != f_iki_syntax_quote_single_s.string[0])) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_1_parameter_checking_

    if (!vocabulary.used) {
      state->status = F_status_set_error(F_data_not);

      return;
    }

    f_range_t range = macro_f_range_t_initialize_2(vocabulary.used);
    f_number_unsigned_t i = 0;

    do {
      if (state->interrupt) {
        state->interrupt((void * const) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) break;
      }

      i = vocabulary.used - range.start;

      state->status = f_utf_is_word_dash_plus(vocabulary.string + range.start, i, F_false);
      if (F_status_is_error(state->status)) break;

      if (state->status == F_false) {
        state->status = F_status_set_error(F_syntax);

        break;
      }

      state->status = f_utf_buffer_increment(vocabulary, &range, 1);

    } while (F_status_is_error_not(state->status) && range.start < range.stop);

    // Pre-allocate the expected size along with two extra slots for slashes to help reduce potential allocations.
    if (F_status_is_error_not(state->status)) {
      state->status = f_memory_array_increase_by(vocabulary.used + content.used + f_iki_syntax_wrap_close_s.used + f_iki_syntax_wrap_open_s.used + f_iki_syntax_separator_s.used + (f_iki_syntax_slash_s.used * 2) + (quote.used * 2), sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
    }

    if (F_status_is_error_not(state->status) && (state->flag & f_iki_state_flag_force_wrap_e)) {
      state->status = f_string_dynamic_append(f_iki_syntax_wrap_open_s, destination);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_string_dynamic_append(vocabulary, destination);
    }

    if (F_status_is_error_not(state->status) && (state->flag & f_iki_state_flag_force_wrap_e)) {
      state->status = f_string_dynamic_append(f_iki_syntax_wrap_close_s, destination);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_string_dynamic_append(f_iki_syntax_separator_s, destination);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_string_dynamic_append(quote, destination);
    }

    if (F_status_is_error_not(state->status)) {
      f_number_unsigned_t total = 0;
      f_number_unsigned_t j = 0;

      range.start = 0;
      range.stop = content.used - 1;

      for (i = 0; i < content.used; ++i) {

        if (!content.string[i]) continue;

        if (content.string[i] == quote.string[0]) {
          state->status = f_memory_array_increase_by((i - range.start) + 1 + (total * f_iki_syntax_slash_s.used) + f_iki_syntax_slash_s.used + quote.used, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

          if (F_status_is_error_not(state->status) && i) {
            range.stop = i - 1;

            state->status = f_string_dynamic_partial_append(content, range, destination);
          }

          if (total) {
            for (j = 0; F_status_is_error_not(state->status) && j < total; ++j) {
              state->status = f_string_dynamic_append(f_iki_syntax_slash_s, destination);
            } // for
          }

          if (F_status_is_error_not(state->status)) {
            state->status = f_string_dynamic_append(f_iki_syntax_slash_s, destination);
          }

          if (F_status_is_error_not(state->status)) {
            state->status = f_string_dynamic_append(quote, destination);
          }

          if (F_status_is_error(state->status)) break;

          range.start = i + 1;
          range.stop = content.used - 1;
          total = 0;
        }
        else if (content.string[i] == f_iki_syntax_slash_s.string[0]) {
          ++total;
        }
        else if (total) {
          total = 0;
        }
      } // for

      if (F_status_is_error_not(state->status) && range.start <= range.stop) {
        state->status = f_string_dynamic_partial_append(content, range, destination);
      }

      // Slashes at the end must be escaped.
      if (F_status_is_error_not(state->status) && total) {
        state->status = f_memory_array_increase_by(total, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

        for (j = 0; F_status_is_error_not(state->status) && j < total; ++j) {
          state->status = f_string_dynamic_append(f_iki_syntax_slash_s, destination);
        } // for
      }
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_string_dynamic_append(quote, destination);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = F_okay;
    }
  }
#endif // _di_f_iki_write_

#ifdef __cplusplus
} // extern "C"
#endif
