/**
 * FLL - Level 3
 *
 * Project: Featureless Make
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides common define types.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _fake_common_define_h
#define _fake_common_define_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The program allocation defines.
 *
 * fake_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   An allocation step used for buffers that are anticipated to have large buffers.
 *   - pipe:    A buffer size used for processing piped data.
 *   - small:   An allocation step used for buffers that are anticipated to have small buffers.
 */
#ifndef _di_fake_allocation_d_
  #define fake_allocation_console_d 4
  #define fake_allocation_large_d   256
  #define fake_allocation_pipe_d    16384
  #define fake_allocation_small_d   16
#endif // _di_fake_allocation_d_

/**
 * Flags passed to the main function or program.
 *
 * fake_main_flag_*_d:
 *   - none:                   No flags set.
 *   - copyright:              Print copyright.
 *   - enable_documents:       Enable documents, as in --enable-doc (not specifying means --disable-doc).
 *   - enable_shared:          Enable shared, as in --enable-shared (not specifying means --disable-shared).
 *   - enable_static:          Enable static, as in --enable-static (not specifying means --disable-static).
 *   - header:                 Enable printing of headers.
 *   - help:                   Print help.
 *   - operation:              Designate that an operation is explicitly passed.
 *   - operation_build:        The build operation is designated.
 *   - operation_clean:        The clean operation is designated.
 *   - operation_make:         The make operation is designated (or enabled as default).
 *   - operation_skeleton:     The skeleton operation is designated.
 *   - pipe:                   Use the input pipe.
 *   - separate:               Enable printing of separators.
 *   - strip_invalid:          Using strip invalid character mode.
 *   - verify:                 Using verify mode.
 *   - version:                Print version.
 *   - version_copyright_help: A helper flag representing version, copyright, and help flag bits being set.
 */
#ifndef _di_fake_main_flag_d_
  #define fake_main_flag_none_d                   0x0
  #define fake_main_flag_copyright_d              0x1
  #define fake_main_flag_enable_documents_d       0x2
  #define fake_main_flag_enable_shared_d          0x4
  #define fake_main_flag_enable_static_d          0x8
  #define fake_main_flag_header_d                 0x10
  #define fake_main_flag_help_d                   0x20
  #define fake_main_flag_operation_d              0x40
  #define fake_main_flag_operation_build_d        0x80
  #define fake_main_flag_operation_clean_d        0x100
  #define fake_main_flag_operation_make_d         0x200
  #define fake_main_flag_operation_skeleton_d     0x400
  #define fake_main_flag_pipe_d                   0x800
  #define fake_main_flag_separate_d               0x1000
  #define fake_main_flag_strip_invalid_d          0x2000
  #define fake_main_flag_verify_d                 0x4000
  #define fake_main_flag_version_d                0x8000
  #define fake_main_flag_version_copyright_help_d 0x8021
#endif // _di_fake_main_flag_d_

/**
 * Provide common/generic definitions.
 *
 * _di_fake_max_*_d:
 *   - initial_buffer:  Max size to maintain the buffer (to avoid buffer bloat after an overly large allocation) (131072 = 128k).
 *   - operation:       Maximum number of operations.
 *   - over_array:      Max size when array is over this to shrink the array when done.
 *   - over_string:     Max size when string is over this to shrink the string when done.
 *   - recursion_depth: Maximum recursion depth.
 *   - stack:           Maximum stack call depth.
 */
#ifndef _di_fake_max_d_
  #define fake_max_initial_buffer_d  131072
  #define fake_max_operation_d       35
  #define fake_max_over_array_d      4096
  #define fake_max_over_string_d     65536
  #define fake_max_recursion_depth_d 65535
  #define fake_max_stack_d           8192
#endif // _di_fake_max_d_

/**
 * Flags for fine-tuned print control.
 *
 * fake_print_flag_*_d:
 *   - none:    No flags set.
 *   - debug:   Stream is for debug printing.
 *   - error:   Stream is for error printing.
 *   - in:      Stream is a source file.
 *   - message: Stream is for message printing.
 *   - out:     Stream is a destination file.
 *   - warning: Stream is for warning printing.
 */
#ifndef _di_fake_print_flag_d_
  #define fake_print_flag_none_d    0x0
  #define fake_print_flag_debug_d   0x1
  #define fake_print_flag_error_d   0x2
  #define fake_print_flag_file_d    0x4
  #define fake_print_flag_in_d      0x8
  #define fake_print_flag_out_d     0x10
  #define fake_print_flag_message_d 0x20
  #define fake_print_flag_warning_d 0x40
#endif // _di_fake_print_flag_d_

/**
 * The program signal defines.
 *
 * fake_signal_*_d:
 *   - check:          Number of iterations before performing signal check in non-threaded signal handling.
 *   - check_failsafe: When using threads, how many consecutive failures to check signal before aborting (as a recursion failsafe).
 *   - check_tiny:     The tiny check.
 *   - check_short:    The short signal check.
 */
#ifndef _di_fake_signal_d_
  #define fake_signal_check_d          500000
  #define fake_signal_check_failsafe_d 20000
  #define fake_signal_check_tiny_d     4
  #define fake_signal_check_short_d    16
#endif // _di_fake_signal_d_

/**
 * Flags for the code property in the f_state_t structure.
 *
 * fake_state_code_*_d:
 *   - none:  No flags set.
 *   - clone: Designate that the operation is a clone operation (as opposed to a copy operation).
 *   - local: Designate that the state.custom is in fake_local_t rather than fake_main_t.
 */
#ifndef _di_fake_state_code_d_
  #define fake_state_code_none_d  0x0
  #define fake_state_code_clone_d 0x1
  #define fake_state_code_local_d 0x2
#endif // _di_fake_state_code_d_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _fake_common_define_h
